@@ -42,16 +42,16 @@
 #include <stdint.h>
 #include <string.h>
 
-#include "cpu.h"
 #include "flow.h"
 
 #include "dpif-netdev-private-dpcls.h"
 #include "dpif-netdev-private-extract.h"
 #include "dpif-netdev-private-flow.h"
+#include "dp-packet.h"
+#include "packets.h"
 
 /* AVX512-BW level permutex2var_epi8 emulation. */
 static inline __m512i
-__attribute__((target("avx512bw")))
 _mm512_maskz_permutex2var_epi8_skx(__mmask64 k_mask,
                                    __m512i v_data_0,
                                    __m512i v_shuf_idxs,
@@ -108,13 +108,46 @@ _mm512_maskz_permutex2var_epi8_skx(__mmask64 k_mask,
     return v_result_kmskd;
 }
 
-/* Wrapper function required to enable ISA. */
+/* Wrapper function to enable VBMI ISA required by the
+ * _mm512_maskz_permutexvar_epi8 intrinsic. */
+#if HAVE_AVX512VBMI
 static inline __m512i
 __attribute__((__target__("avx512vbmi")))
 _mm512_maskz_permutexvar_epi8_wrap(__mmask64 kmask, __m512i idx, __m512i a)
 {
     return _mm512_maskz_permutexvar_epi8(kmask, idx, a);
 }
+#endif
+
+static inline __m512i
+_mm512_maskz_permutexvar_epi8_selector(__mmask64 k_shuf, __m512i v_shuf,
+                                       __m512i v_pkt0,
+                                       const uint32_t use_vbmi OVS_UNUSED)
+{
+    /* Permute the packet layout into miniflow blocks shape. */
+    __m512i v512_zeros = _mm512_setzero_si512();
+    __m512i v_blk0;
+#if HAVE_AVX512VBMI
+#if __GNUC__ >= 4
+    if (__builtin_constant_p(use_vbmi) && use_vbmi) {
+#else
+    if (use_vbmi) {
+#endif
+        /* As different AVX512 ISA levels have different implementations,
+        * this specializes on the use_vbmi attribute passed in.
+        */
+        v_blk0 = _mm512_maskz_permutexvar_epi8_wrap(k_shuf, v_shuf, v_pkt0);
+
+    } else {
+        v_blk0 = _mm512_maskz_permutex2var_epi8_skx(k_shuf, v_pkt0, v_shuf,
+                                                    v512_zeros);
+    }
+#else
+    v_blk0 = _mm512_maskz_permutex2var_epi8_skx(k_shuf, v_pkt0, v_shuf,
+                                                v512_zeros);
+#endif
+    return v_blk0;
+}
 
 
 /* This file contains optimized implementations of miniflow_extract()
@@ -137,6 +170,7 @@ _mm512_maskz_permutexvar_epi8_wrap(__mmask64 kmask, __m512i idx, __m512i a)
 #define PATTERN_ETHERTYPE_MASK PATTERN_ETHERTYPE_GEN(0xFF, 0xFF)
 #define PATTERN_ETHERTYPE_IPV4 PATTERN_ETHERTYPE_GEN(0x08, 0x00)
 #define PATTERN_ETHERTYPE_DT1Q PATTERN_ETHERTYPE_GEN(0x81, 0x00)
+#define PATTERN_ETHERTYPE_IPV6 PATTERN_ETHERTYPE_GEN(0x86, 0xDD)
 
 /* VLAN (Dot1Q) patterns and masks. */
 #define PATTERN_DT1Q_MASK                                               \
@@ -160,6 +194,7 @@ _mm512_maskz_permutexvar_epi8_wrap(__mmask64 kmask, __m512i idx, __m512i a)
 #define PATTERN_IPV4_MASK PATTERN_IPV4_GEN(0xFF, 0xBF, 0xFF, 0xFF)
 #define PATTERN_IPV4_UDP PATTERN_IPV4_GEN(0x45, 0, 0, 0x11)
 #define PATTERN_IPV4_TCP PATTERN_IPV4_GEN(0x45, 0, 0, 0x06)
+#define PATTERN_IPV4_NVGRE PATTERN_IPV4_GEN(0x45, 0, 0, 0x2f)
 
 #define PATTERN_TCP_GEN(data_offset)                                    \
   0, 0, 0, 0, /* sport, dport */                                        \
@@ -184,6 +219,12 @@ _mm512_maskz_permutexvar_epi8_wrap(__mmask64 kmask, __m512i idx, __m512i a)
   NU, NU, NU, NU, NU, NU, NU, NU, 34, 35, 36, 37, NU, NU, NU, NU, /* TCP */   \
   NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, /* Unused. */
 
+#define PATTERN_IPV4_NVGRE_SHUFFLE \
+   0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, NU, NU, /* Ether */ \
+  26, 27, 28, 29, 30, 31, 32, 33, NU, NU, NU, NU, 20, 15, 22, 23, /* IPv4 */  \
+  NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, /* Unused */\
+  NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, /* Unused */
+
 #define PATTERN_DT1Q_IPV4_UDP_SHUFFLE                                         \
   /* Ether (2 blocks): Note that *VLAN* type is written here. */              \
   0,  1,  2,  3,  4,  5,  6,  7, 8,  9, 10, 11, 16, 17,  0,  0,               \
@@ -201,6 +242,40 @@ _mm512_maskz_permutexvar_epi8_wrap(__mmask64 kmask, __m512i idx, __m512i a)
   NU, NU, NU, NU, NU, NU, NU, NU, 38, 39, 40, 41, NU, NU, NU, NU, /* TCP */   \
   NU, NU, NU, NU, NU, NU, NU, NU, /* Unused. */
 
+/* Generator for checking IPv6 ver. */
+#define PATTERN_IPV6_GEN(VER_TRC, PROTO)                                      \
+  VER_TRC,     /* Version: 4bits and Traffic class: 4bits. */                 \
+  0, 0, 0,     /* Traffic class: 4bits and Flow Label: 24bits. */             \
+  0, 0,        /* Payload length 16bits. */                                   \
+  PROTO, 0,    /* Next Header 8bits and Hop limit 8bits. */                   \
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* Src IP: 128bits. */      \
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* Dst IP: 128bits. */
+
+#define PATTERN_IPV6_MASK PATTERN_IPV6_GEN(0xF0, 0xFF)
+#define PATTERN_IPV6_UDP PATTERN_IPV6_GEN(0x60, 0x11)
+#define PATTERN_IPV6_TCP PATTERN_IPV6_GEN(0x60, 0x06)
+
+#define PATTERN_IPV6_SHUFFLE                                                  \
+   0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, NU, NU, /* Ether */ \
+  22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, /* IPv6 */  \
+  38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, /* IPv6 */  \
+  NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, NU, /* Unused */
+
+/* VLAN (Dot1Q) patterns and masks. */
+#define PATTERN_DT1Q_MASK                                                     \
+  0x00, 0x00, 0xFF, 0xFF,
+#define PATTERN_DT1Q_IPV6                                                     \
+  0x00, 0x00, 0x86, 0xDD,
+
+#define PATTERN_DT1Q_IPV6_SHUFFLE                                             \
+  /* Ether (2 blocks): Note that *VLAN* type is written here. */              \
+  0,  1,  2,  3,  4,  5,  6,  7, 8,  9, 10, 11, 16, 17,  0,  0,               \
+  /* VLAN (1 block): Note that the *EtherHdr->Type* is written here. */       \
+  12, 13, 14, 15, 0, 0, 0, 0,                                                 \
+  26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, /* IPv6 */  \
+  42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, /* IPv6 */  \
+  NU, NU, NU, NU, NU, NU, NU, NU,                                 /* Unused */
+
 /* Generation of K-mask bitmask values, to zero out data in result. Note that
  * these correspond 1:1 to the above "*_SHUFFLE" values, and bit used must be
  * set in this K-mask, and "NU" values must be zero in the k-mask. Each mask
@@ -213,6 +288,13 @@ _mm512_maskz_permutexvar_epi8_wrap(__mmask64 kmask, __m512i idx, __m512i a)
 #define KMASK_IPV4      0xF0FFULL
 #define KMASK_UDP       0x000FULL
 #define KMASK_TCP       0x0F00ULL
+#define KMASK_IPV6      0xFFFFULL
+#define KMASK_ETHER_IPV6     0x3FFFULL
+#define KMASK_DT1Q_IPV6      0xFF0FULL
+#define KMASK_IPV6_NOHDR     0x00FFULL
+
+#define PATTERN_IPV4_KMASK \
+    (KMASK_ETHER | (KMASK_IPV4 << 16))
 
 #define PATTERN_IPV4_UDP_KMASK \
     (KMASK_ETHER | (KMASK_IPV4 << 16) | (KMASK_UDP << 32))
@@ -226,6 +308,13 @@ _mm512_maskz_permutexvar_epi8_wrap(__mmask64 kmask, __m512i idx, __m512i a)
 #define PATTERN_DT1Q_IPV4_TCP_KMASK \
     (KMASK_ETHER | (KMASK_DT1Q << 16) | (KMASK_IPV4 << 24) | (KMASK_TCP << 40))
 
+#define PATTERN_IPV6_KMASK \
+    (KMASK_ETHER_IPV6 | (KMASK_IPV6 << 16) | (KMASK_IPV6 << 32))
+
+#define PATTERN_DT1Q_IPV6_KMASK \
+    (KMASK_ETHER_IPV6 | (KMASK_DT1Q_IPV6 << 16) | (KMASK_IPV6 << 32) | \
+    (KMASK_IPV6_NOHDR << 48))
+
 /* Miniflow Strip post-processing masks.
  * This allows unsetting specific bits from the resulting miniflow. It is used
  * for e.g. IPv4 where the "DF" bit is never pushed to the miniflow itself.
@@ -245,6 +334,49 @@ _mm512_maskz_permutexvar_epi8_wrap(__mmask64 kmask, __m512i idx, __m512i a)
     NC, NC, NC, NC, 0xBF, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC,   \
     NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC
 
+#define PKT_OFFSET_L2_PAD_SIZE    (ETH_HEADER_LEN)
+#define PKT_OFFSET_L3             (ETH_HEADER_LEN)
+#define PKT_OFFSET_VLAN_L3        (ETH_HEADER_LEN + VLAN_HEADER_LEN)
+#define PKT_OFFSET_IPV4_L4        (ETH_HEADER_LEN + IP_HEADER_LEN)
+#define PKT_OFFSET_VLAN_IPV4_L4   (PKT_OFFSET_IPV4_L4 + VLAN_HEADER_LEN)
+#define PKT_OFFSET_VLAN_IPV6_L4   (PKT_OFFSET_VLAN_L3 + IPV6_HEADER_LEN)
+#define PKT_OFFSET_IPV6_L4        (PKT_OFFSET_L3 + IPV6_HEADER_LEN)
+
+#define PKT_MIN_ETH_IPV4          (ETH_HEADER_LEN + IP_HEADER_LEN)
+#define PKT_MIN_ETH_IPV4_UDP      (PKT_OFFSET_IPV4_L4 + UDP_HEADER_LEN)
+#define PKT_MIN_ETH_VLAN_IPV4_UDP (PKT_OFFSET_VLAN_IPV4_L4 + UDP_HEADER_LEN)
+#define PKT_MIN_ETH_IPV4_TCP      (PKT_OFFSET_IPV4_L4 + TCP_HEADER_LEN)
+#define PKT_MIN_ETH_VLAN_IPV4_TCP (PKT_OFFSET_VLAN_IPV4_L4 + TCP_HEADER_LEN)
+#define PKT_MIN_ETH_IPV6_UDP      (PKT_OFFSET_IPV6_L4 + UDP_HEADER_LEN)
+#define PKT_MIN_ETH_VLAN_IPV6_UDP (PKT_OFFSET_VLAN_IPV6_L4 + UDP_HEADER_LEN)
+#define PKT_MIN_ETH_IPV6_TCP      (PKT_OFFSET_IPV6_L4 + TCP_HEADER_LEN)
+#define PKT_MIN_ETH_VLAN_IPV6_TCP (PKT_OFFSET_VLAN_IPV6_L4 + TCP_HEADER_LEN)
+
+/* MF bits. */
+#define MF_BIT(field) (MAP_1 << ((offsetof(struct flow, field) / 8) %         \
+                       MAP_T_BITS))
+#define MF_WORD(field, n_word)                                                \
+            (((MAP_1 << n_word) - 1) << ((offsetof(struct flow, field) / 8) % \
+             MAP_T_BITS))
+
+#define MF_ETH        (MF_BIT(dp_hash) | MF_BIT(in_port) | MF_BIT(packet_type)\
+                       | MF_BIT(dl_dst) | MF_BIT(dl_src)| MF_BIT(dl_type))
+#define MF_ETH_VLAN   (MF_ETH | MF_BIT(vlans))
+
+#define MF_IPV4       (MF_BIT(nw_src) | MF_BIT(ipv6_label))
+#define MF_IPV4_UDP   (MF_IPV4 | MF_BIT(tp_src) | MF_BIT(tp_dst))
+#define MF_IPV4_TCP   (MF_IPV4_UDP | MF_BIT(tcp_flags) | MF_BIT(arp_tha.ea[2]))
+
+#define MF_IPV6_UDP   (MF_BIT(ipv6_label) | MF_WORD(ipv6_src, 2) |            \
+                       MF_WORD(ipv6_dst, 2) | MF_BIT(tp_src) | MF_BIT(tp_dst))
+#define MF_IPV6_TCP   (MF_IPV6_UDP | MF_BIT(tcp_flags) | MF_BIT(arp_tha.ea[2]))
+
+#define PATTERN_STRIP_IPV6_MASK                                         \
+    NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC,     \
+    NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC,     \
+    NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC,     \
+    NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC, NC
+
 /* This union allows initializing static data as u8, but easily loading it
  * into AVX512 registers too. The union ensures proper alignment for the zmm.
  */
@@ -324,6 +456,11 @@ enum MFEX_PROFILES {
     PROFILE_ETH_IPV4_TCP,
     PROFILE_ETH_VLAN_IPV4_UDP,
     PROFILE_ETH_VLAN_IPV4_TCP,
+    PROFILE_ETH_IPV6_UDP,
+    PROFILE_ETH_IPV6_TCP,
+    PROFILE_ETH_VLAN_IPV6_TCP,
+    PROFILE_ETH_VLAN_IPV6_UDP,
+    PROFILE_ETH_IPV4_NVGRE,
     PROFILE_COUNT,
 };
 
@@ -342,11 +479,11 @@ static const struct mfex_profile mfex_profiles[PROFILE_COUNT] =
         .strip_mask.u8_data = { PATTERN_STRIP_IPV4_MASK },
         .store_kmsk = PATTERN_IPV4_UDP_KMASK,
 
-        .mf_bits = { 0x18a0000000000000, 0x0000000000040401},
+        .mf_bits = { MF_ETH, MF_IPV4_UDP},
         .dp_pkt_offs = {
-            0, UINT16_MAX, 14, 34,
+            0, UINT16_MAX, PKT_OFFSET_L3, PKT_OFFSET_IPV4_L4,
         },
-        .dp_pkt_min_size = 42,
+        .dp_pkt_min_size = PKT_MIN_ETH_IPV4_UDP,
     },
 
     [PROFILE_ETH_IPV4_TCP] = {
@@ -365,11 +502,11 @@ static const struct mfex_profile mfex_profiles[PROFILE_COUNT] =
         .strip_mask.u8_data = { PATTERN_STRIP_IPV4_MASK },
         .store_kmsk = PATTERN_IPV4_TCP_KMASK,
 
-        .mf_bits = { 0x18a0000000000000, 0x0000000000044401},
+        .mf_bits = { MF_ETH, MF_IPV4_TCP},
         .dp_pkt_offs = {
-            0, UINT16_MAX, 14, 34,
+            0, UINT16_MAX, PKT_OFFSET_L3, PKT_OFFSET_IPV4_L4,
         },
-        .dp_pkt_min_size = 54,
+        .dp_pkt_min_size = PKT_MIN_ETH_IPV4_TCP,
     },
 
     [PROFILE_ETH_VLAN_IPV4_UDP] = {
@@ -384,11 +521,12 @@ static const struct mfex_profile mfex_profiles[PROFILE_COUNT] =
         .strip_mask.u8_data = { PATTERN_STRIP_DOT1Q_IPV4_MASK },
         .store_kmsk = PATTERN_DT1Q_IPV4_UDP_KMASK,
 
-        .mf_bits = { 0x38a0000000000000, 0x0000000000040401},
+        .mf_bits = { MF_ETH_VLAN, MF_IPV4_UDP},
         .dp_pkt_offs = {
-            14, UINT16_MAX, 18, 38,
+            PKT_OFFSET_L2_PAD_SIZE, UINT16_MAX, PKT_OFFSET_VLAN_L3,
+            PKT_OFFSET_VLAN_IPV4_L4,
         },
-        .dp_pkt_min_size = 46,
+        .dp_pkt_min_size = PKT_MIN_ETH_VLAN_IPV4_UDP,
     },
 
     [PROFILE_ETH_VLAN_IPV4_TCP] = {
@@ -409,14 +547,159 @@ static const struct mfex_profile mfex_profiles[PROFILE_COUNT] =
         .strip_mask.u8_data = { PATTERN_STRIP_DOT1Q_IPV4_MASK },
         .store_kmsk = PATTERN_DT1Q_IPV4_TCP_KMASK,
 
-        .mf_bits = { 0x38a0000000000000, 0x0000000000044401},
+        .mf_bits = { MF_ETH_VLAN, MF_IPV4_TCP},
         .dp_pkt_offs = {
-            14, UINT16_MAX, 18, 38,
+            PKT_OFFSET_L2_PAD_SIZE, UINT16_MAX, PKT_OFFSET_VLAN_L3,
+            PKT_OFFSET_VLAN_IPV4_L4,
         },
-        .dp_pkt_min_size = 58,
+        .dp_pkt_min_size = PKT_MIN_ETH_VLAN_IPV4_TCP,
+    },
+
+    [PROFILE_ETH_IPV6_UDP] = {
+        .probe_mask.u8_data = { PATTERN_ETHERTYPE_MASK PATTERN_IPV6_MASK },
+        .probe_data.u8_data = { PATTERN_ETHERTYPE_IPV6 PATTERN_IPV6_UDP },
+
+        .store_shuf.u8_data = { PATTERN_IPV6_SHUFFLE },
+        .strip_mask.u8_data = { PATTERN_STRIP_IPV6_MASK },
+        .store_kmsk = PATTERN_IPV6_KMASK,
+
+        .mf_bits = { MF_ETH, MF_IPV6_UDP},
+        .dp_pkt_offs = {
+            0, UINT16_MAX, PKT_OFFSET_L3, PKT_OFFSET_IPV6_L4,
+        },
+        .dp_pkt_min_size = PKT_MIN_ETH_IPV6_UDP,
+    },
+
+    [PROFILE_ETH_IPV6_TCP] = {
+        .probe_mask.u8_data = { PATTERN_ETHERTYPE_MASK PATTERN_IPV6_MASK },
+        .probe_data.u8_data = { PATTERN_ETHERTYPE_IPV6 PATTERN_IPV6_TCP },
+
+        .store_shuf.u8_data = { PATTERN_IPV6_SHUFFLE },
+        .strip_mask.u8_data = { PATTERN_STRIP_IPV6_MASK },
+        .store_kmsk = PATTERN_IPV6_KMASK,
+
+        .mf_bits = { MF_ETH, MF_IPV6_TCP},
+        .dp_pkt_offs = {
+            0, UINT16_MAX, PKT_OFFSET_L3, PKT_OFFSET_IPV6_L4,
+        },
+        .dp_pkt_min_size = PKT_MIN_ETH_IPV6_TCP,
+    },
+
+    [PROFILE_ETH_VLAN_IPV6_TCP] = {
+        .probe_mask.u8_data = {
+            PATTERN_ETHERTYPE_MASK PATTERN_DT1Q_MASK PATTERN_IPV6_MASK },
+        .probe_data.u8_data = {
+            PATTERN_ETHERTYPE_DT1Q PATTERN_DT1Q_IPV6 PATTERN_IPV6_TCP },
+
+        .store_shuf.u8_data = { PATTERN_DT1Q_IPV6_SHUFFLE },
+        .strip_mask.u8_data = { PATTERN_STRIP_IPV6_MASK },
+        .store_kmsk = PATTERN_DT1Q_IPV6_KMASK,
+
+        .mf_bits = { MF_ETH_VLAN, MF_IPV6_TCP},
+        .dp_pkt_offs = {
+            PKT_OFFSET_L2_PAD_SIZE, UINT16_MAX, PKT_OFFSET_VLAN_L3,
+            PKT_OFFSET_VLAN_IPV6_L4,
+        },
+        .dp_pkt_min_size = PKT_MIN_ETH_VLAN_IPV6_TCP,
+    },
+
+    [PROFILE_ETH_VLAN_IPV6_UDP] = {
+        .probe_mask.u8_data = {
+            PATTERN_ETHERTYPE_MASK PATTERN_DT1Q_MASK PATTERN_IPV6_MASK },
+        .probe_data.u8_data = {
+            PATTERN_ETHERTYPE_DT1Q PATTERN_DT1Q_IPV6 PATTERN_IPV6_UDP },
+
+        .store_shuf.u8_data = { PATTERN_DT1Q_IPV6_SHUFFLE },
+        .strip_mask.u8_data = { PATTERN_STRIP_IPV6_MASK },
+        .store_kmsk = PATTERN_DT1Q_IPV6_KMASK,
+
+        .mf_bits = { MF_ETH_VLAN, MF_IPV6_UDP},
+        .dp_pkt_offs = {
+            PKT_OFFSET_L2_PAD_SIZE, UINT16_MAX, PKT_OFFSET_VLAN_L3,
+            PKT_OFFSET_VLAN_IPV6_L4,
+        },
+        .dp_pkt_min_size = PKT_MIN_ETH_VLAN_IPV6_UDP,
+    },
+
+    [PROFILE_ETH_IPV4_NVGRE] = {
+        .probe_mask.u8_data = { PATTERN_ETHERTYPE_MASK PATTERN_IPV4_MASK },
+        .probe_data.u8_data = { PATTERN_ETHERTYPE_IPV4 PATTERN_IPV4_NVGRE},
+
+        .store_shuf.u8_data = { PATTERN_IPV4_NVGRE_SHUFFLE },
+        .strip_mask.u8_data = { PATTERN_STRIP_IPV4_MASK },
+        .store_kmsk = PATTERN_IPV4_KMASK,
+
+        .mf_bits = { MF_ETH, MF_IPV4},
+        .dp_pkt_offs = {
+            0, UINT16_MAX, PKT_OFFSET_L3, PKT_OFFSET_IPV4_L4,
+        },
+        .dp_pkt_min_size = PKT_MIN_ETH_IPV4,
     },
 };
 
+/* IPv6 header helper function to fix TC, flow label and next header. */
+static inline void ALWAYS_INLINE
+mfex_handle_ipv6_hdr_block(const uint8_t *ipv6, uint64_t *block)
+{
+    static const uint8_t data_shuf[16] = {
+        0, 1, 2, 3, /* copy IPv6 label in place, it is masked later. */
+        1, 0,       /* Byte-swap TC fields for LE usage. */
+        7, 6,       /* Move TTL and next proto to MF required locations. */
+    };
+
+    /* BE mask for IPv6 label, and mask to strip away unwanted TC bits. */
+    const uint64_t mask = 0xffff0f00 | (UINT64_MAX << 40);
+    uint64_t mask_data[2] = { mask, mask };
+
+    /* Load constant data. Is lifted to occur 1x per burst, not per packet. */
+    __m128i ipv6_hdr = _mm_loadu_si128((void *) ipv6);
+    __m128i v_mask = _mm_loadu_si128((void *) mask_data);
+    __m128i v_shuf_mask = _mm_loadu_si128((void *) data_shuf);
+
+    /* Shuffle data layout, shift 16-bits to get TC fixed, mask to cleanup. */
+    __m128i v_ipv6 = _mm_shuffle_epi8(ipv6_hdr, v_shuf_mask);
+    __m128i v_tc_shift = _mm_mask_slli_epi16(v_ipv6, 0b100, v_ipv6, 4);
+    __m128i v_ipv6_m = _mm_and_si128(v_tc_shift, v_mask);
+
+    *block = _mm_extract_epi64(v_ipv6_m, 0);
+}
+
+/* IPv6 Protocol specific helper functions, for handling L4 UDP/TCP. */
+static inline void
+mfex_handle_ipv6_l4(const uint8_t *ports, uint64_t *block)
+{
+    memcpy(block, ports, sizeof(uint32_t));
+}
+
+/* IPv6 specific helper functions, for calculating offsets/lengths. */
+static int
+mfex_ipv6_set_l2_pad_size(struct dp_packet *pkt,
+                          struct ovs_16aligned_ip6_hdr *nh,
+                          uint32_t len_from_ipv6,
+                          uint32_t next_hdr_size)
+{
+    /* Handle dynamic l2_pad_size. */
+    uint16_t p_len =  ntohs(nh->ip6_ctlun.ip6_un1.ip6_un1_plen);
+
+    /* Error if IP total length is greater than remaining packet size. */
+    bool err_ipv6_len_too_high = p_len + IPV6_HEADER_LEN > len_from_ipv6;
+
+    /* Plen must be greater then the l4 packet header. */
+    bool err_ipv6_len_too_low = p_len < next_hdr_size;
+
+    bool err_packet_size_low = len_from_ipv6 < sizeof *nh;
+
+    /* Ensure the l2 pad size will not overflow. */
+    bool err_len_u16_overflow = (len_from_ipv6 - (p_len + IPV6_HEADER_LEN))
+                                > UINT16_MAX;
+
+    if (OVS_UNLIKELY(err_ipv6_len_too_high || err_ipv6_len_too_low ||
+                     err_len_u16_overflow || err_packet_size_low)) {
+        return -1;
+    }
+    dp_packet_set_l2_pad_size(pkt, len_from_ipv6 - (p_len + IPV6_HEADER_LEN));
+    return 0;
+}
 
 /* Protocol specific helper functions, for calculating offsets/lenghts. */
 static int32_t
@@ -467,6 +750,14 @@ mfex_handle_tcp_flags(const struct tcp_header *tcp, uint64_t *block)
     *block = ctl_u64 << 32;
 }
 
+static int
+mfex_check_tcp_data_offset(const struct tcp_header *tcp)
+{
+    /* we dont support TCP options, offset must be 5. */
+    bool ret = TCP_OFFSET(tcp->tcp_ctl) == 5;
+    return ret;
+}
+
 /* Generic loop to process any mfex profile. This code is specialized into
  * multiple actual MFEX implementation functions. Its marked ALWAYS_INLINE
  * to ensure the compiler specializes each instance. The code is marked "hot"
@@ -481,7 +772,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                     odp_port_t in_port,
                     void *pmd_handle OVS_UNUSED,
                     const enum MFEX_PROFILES profile_id,
-                    const uint32_t use_vbmi)
+                    const uint32_t use_vbmi OVS_UNUSED)
 {
     uint32_t hitmask = 0;
     struct dp_packet *packet;
@@ -538,19 +829,9 @@ mfex_avx512_process(struct dp_packet_batch *packets,
         _mm_storeu_si128((void *) bits, v_bits);
         _mm_storeu_si128((void *) blocks, v_blocks01);
 
-        /* Permute the packet layout into miniflow blocks shape.
-         * As different AVX512 ISA levels have different implementations,
-         * this specializes on the "use_vbmi" attribute passed in.
-         */
-        __m512i v512_zeros = _mm512_setzero_si512();
-        __m512i v_blk0;
-        if (__builtin_constant_p(use_vbmi) && use_vbmi) {
-            v_blk0 = _mm512_maskz_permutexvar_epi8_wrap(k_shuf, v_shuf,
-                                                        v_pkt0);
-        } else {
-            v_blk0 = _mm512_maskz_permutex2var_epi8_skx(k_shuf, v_pkt0,
-                                                        v_shuf, v512_zeros);
-        }
+        __m512i v_blk0 = _mm512_maskz_permutexvar_epi8_selector(k_shuf, v_shuf,
+                                                                v_pkt0,
+                                                                use_vbmi);
 
         __m512i v_blk0_strip = _mm512_and_si512(v_blk0, v_strp);
         _mm512_storeu_si512(&blocks[2], v_blk0_strip);
@@ -577,6 +858,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 /* Process TCP flags, and store to blocks. */
                 const struct tcp_header *tcp = (void *)&pkt[38];
                 mfex_handle_tcp_flags(tcp, &blocks[7]);
+                dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
             } break;
 
         case PROFILE_ETH_VLAN_IPV4_UDP: {
@@ -588,6 +870,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                                               UDP_HEADER_LEN)) {
                     continue;
                 }
+                dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
             } break;
 
         case PROFILE_ETH_IPV4_TCP: {
@@ -602,6 +885,7 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                                               TCP_HEADER_LEN)) {
                     continue;
                 }
+                dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
             } break;
 
         case PROFILE_ETH_IPV4_UDP: {
@@ -612,24 +896,122 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                                               UDP_HEADER_LEN)) {
                     continue;
                 }
-
+                dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
             } break;
+
+        case PROFILE_ETH_IPV6_UDP: {
+                /* Handle dynamic l2_pad_size. */
+                uint32_t size_from_ipv6 = size - sizeof(struct eth_header);
+                struct ovs_16aligned_ip6_hdr *nh = (void *)&pkt[sizeof
+                                                   (struct eth_header)];
+                if (mfex_ipv6_set_l2_pad_size(packet, nh, size_from_ipv6,
+                                              UDP_HEADER_LEN)) {
+                    continue;
+                }
+
+                /* Process IPv6 header for TC, flow Label and next header. */
+                mfex_handle_ipv6_hdr_block(&pkt[ETH_HEADER_LEN], &blocks[8]);
+
+                /* Process UDP header. */
+                mfex_handle_ipv6_l4((void *)&pkt[54], &blocks[9]);
+                dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
+            } break;
+
+        case PROFILE_ETH_IPV6_TCP: {
+                /* Handle dynamic l2_pad_size. */
+                uint32_t size_from_ipv6 = size - sizeof(struct eth_header);
+                struct ovs_16aligned_ip6_hdr *nh = (void *)&pkt[sizeof
+                                                   (struct eth_header)];
+                if (mfex_ipv6_set_l2_pad_size(packet, nh, size_from_ipv6,
+                                              TCP_HEADER_LEN)) {
+                    continue;
+                }
+
+                /* Process IPv6 header for TC, flow Label and next header. */
+                mfex_handle_ipv6_hdr_block(&pkt[ETH_HEADER_LEN], &blocks[8]);
+
+                /* Process TCP header. */
+                mfex_handle_ipv6_l4((void *)&pkt[54], &blocks[10]);
+                const struct tcp_header *tcp = (void *)&pkt[54];
+                if (!mfex_check_tcp_data_offset(tcp)) {
+                    continue;
+                }
+                mfex_handle_tcp_flags(tcp, &blocks[9]);
+                dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
+            } break;
+
+        case PROFILE_ETH_VLAN_IPV6_TCP: {
+                mfex_vlan_pcp(pkt[14], &keys[i].buf[4]);
+
+                /* Handle dynamic l2_pad_size. */
+                uint32_t size_from_ipv6 = size - VLAN_ETH_HEADER_LEN;
+                struct ovs_16aligned_ip6_hdr *nh = (void *)&pkt
+                                                   [VLAN_ETH_HEADER_LEN];
+                if (mfex_ipv6_set_l2_pad_size(packet, nh, size_from_ipv6,
+                                              TCP_HEADER_LEN)) {
+                    continue;
+                }
+
+                /* Process IPv6 header for TC, flow Label and next header. */
+                mfex_handle_ipv6_hdr_block(&pkt[VLAN_ETH_HEADER_LEN],
+                                           &blocks[9]);
+
+                /* Process TCP header. */
+                mfex_handle_ipv6_l4((void *)&pkt[58], &blocks[11]);
+                const struct tcp_header *tcp = (void *)&pkt[58];
+                if (!mfex_check_tcp_data_offset(tcp)) {
+                    continue;
+                }
+                mfex_handle_tcp_flags(tcp, &blocks[10]);
+                dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
+            } break;
+
+        case PROFILE_ETH_VLAN_IPV6_UDP: {
+                mfex_vlan_pcp(pkt[14], &keys[i].buf[4]);
+
+                /* Handle dynamic l2_pad_size. */
+                uint32_t size_from_ipv6 = size - VLAN_ETH_HEADER_LEN;
+                struct ovs_16aligned_ip6_hdr *nh = (void *)&pkt
+                                                   [VLAN_ETH_HEADER_LEN];
+                if (mfex_ipv6_set_l2_pad_size(packet, nh, size_from_ipv6,
+                                              UDP_HEADER_LEN)) {
+                    continue;
+                }
+
+                /* Process IPv6 header for TC, flow Label and next header. */
+                mfex_handle_ipv6_hdr_block(&pkt[VLAN_ETH_HEADER_LEN],
+                                           &blocks[9]);
+
+                /* Process UDP header. */
+                mfex_handle_ipv6_l4((void *)&pkt[58], &blocks[10]);
+                dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
+            } break;
+
+        case PROFILE_ETH_IPV4_NVGRE: {
+                /* Handle dynamic l2_pad_size. */
+                uint32_t size_from_ipv4 = size - sizeof(struct eth_header);
+                struct ip_header *nh = (void *)&pkt[sizeof(struct eth_header)];
+                if (mfex_ipv4_set_l2_pad_size(packet, nh, size_from_ipv4, 0)) {
+                    continue;
+                }
+                dp_packet_update_rss_hash_ipv4(packet);
+            } break;
+
         default:
             break;
         };
 
         /* This packet has its miniflow created, add to hitmask. */
-        hitmask |= 1 << i;
+        hitmask |= UINT32_C(1) << i;
     }
 
     return hitmask;
 }
 
 
-#define DECLARE_MFEX_FUNC(name, profile)                                \
+#if HAVE_AVX512VBMI
+#define VBMI_MFEX_FUNC(name, profile)                                   \
 uint32_t                                                                \
-__attribute__((__target__("avx512f")))                                  \
-__attribute__((__target__("avx512vl")))                                 \
 __attribute__((__target__("avx512vbmi")))                               \
 mfex_avx512_vbmi_##name(struct dp_packet_batch *packets,                \
                         struct netdev_flow_key *keys, uint32_t keys_size,\
@@ -638,11 +1020,13 @@ mfex_avx512_vbmi_##name(struct dp_packet_batch *packets,                \
 {                                                                       \
     return mfex_avx512_process(packets, keys, keys_size, in_port,       \
                                pmd_handle, profile, 1);                 \
-}                                                                       \
-                                                                        \
+}
+#else
+#define VBMI_MFEX_FUNC(name, profile)
+#endif
+
+#define BASIC_MFEX_FUNC(name, profile)                                  \
 uint32_t                                                                \
-__attribute__((__target__("avx512f")))                                  \
-__attribute__((__target__("avx512vl")))                                 \
 mfex_avx512_##name(struct dp_packet_batch *packets,                     \
                    struct netdev_flow_key *keys, uint32_t keys_size,    \
                    odp_port_t in_port, struct dp_netdev_pmd_thread      \
@@ -652,6 +1036,10 @@ mfex_avx512_##name(struct dp_packet_batch *packets,                     \
                                pmd_handle, profile, 0);                 \
 }
 
+#define DECLARE_MFEX_FUNC(name, profile)                                \
+VBMI_MFEX_FUNC(name, profile)                                           \
+BASIC_MFEX_FUNC(name, profile)                                          \
+
 /* Each profile gets a single declare here, which specializes the function
  * as required.
  */
@@ -659,47 +1047,11 @@ DECLARE_MFEX_FUNC(ip_udp, PROFILE_ETH_IPV4_UDP)
 DECLARE_MFEX_FUNC(ip_tcp, PROFILE_ETH_IPV4_TCP)
 DECLARE_MFEX_FUNC(dot1q_ip_udp, PROFILE_ETH_VLAN_IPV4_UDP)
 DECLARE_MFEX_FUNC(dot1q_ip_tcp, PROFILE_ETH_VLAN_IPV4_TCP)
-
-
-static int32_t
-avx512_isa_probe(uint32_t needs_vbmi)
-{
-    static enum ovs_cpu_isa isa_required[] = {
-        OVS_CPU_ISA_X86_AVX512F,
-        OVS_CPU_ISA_X86_AVX512BW,
-        OVS_CPU_ISA_X86_BMI2,
-    };
-
-    int32_t ret = 0;
-    for (uint32_t i = 0; i < ARRAY_SIZE(isa_required); i++) {
-        if (!cpu_has_isa(isa_required[i])) {
-            ret = -ENOTSUP;
-        }
-    }
-
-    if (needs_vbmi) {
-        if (!cpu_has_isa(OVS_CPU_ISA_X86_AVX512VBMI)) {
-            ret = -ENOTSUP;
-        }
-    }
-
-    return ret;
-}
-
-/* Probe functions to check ISA requirements. */
-int32_t
-mfex_avx512_probe(void)
-{
-    const uint32_t needs_vbmi = 0;
-    return avx512_isa_probe(needs_vbmi);
-}
-
-int32_t
-mfex_avx512_vbmi_probe(void)
-{
-    const uint32_t needs_vbmi = 1;
-    return avx512_isa_probe(needs_vbmi);
-}
+DECLARE_MFEX_FUNC(ipv6_udp, PROFILE_ETH_IPV6_UDP)
+DECLARE_MFEX_FUNC(ipv6_tcp, PROFILE_ETH_IPV6_TCP)
+DECLARE_MFEX_FUNC(dot1q_ipv6_tcp, PROFILE_ETH_VLAN_IPV6_TCP)
+DECLARE_MFEX_FUNC(dot1q_ipv6_udp, PROFILE_ETH_VLAN_IPV6_UDP)
+DECLARE_MFEX_FUNC(ip_nvgre, PROFILE_ETH_IPV4_NVGRE)
 
 #endif /* __CHECKER__ */
 #endif /* __x86_64__ */
