@@ -119,6 +119,7 @@ nl_sock_create(int protocol, struct nl_sock **sockp)
     struct nl_sock *sock;
 #ifndef _WIN32
     struct sockaddr_nl local, remote;
+    int one = 1;
 #endif
     socklen_t local_size;
     int rcvbuf;
@@ -189,6 +190,11 @@ nl_sock_create(int protocol, struct nl_sock **sockp)
         goto error;
     }
 #else
+    if (setsockopt(sock->fd, SOL_NETLINK, NETLINK_EXT_ACK, &one, sizeof one)) {
+        VLOG_WARN_RL(&rl, "setting extended ack support failed (%s)",
+                     ovs_strerror(errno));
+    }
+
     if (setsockopt(sock->fd, SOL_SOCKET, SO_RCVBUFFORCE,
                    &rcvbuf, sizeof rcvbuf)) {
         /* Only root can use SO_RCVBUFFORCE.  Everyone else gets EPERM.
@@ -910,15 +916,17 @@ nl_sock_transact_multiple__(struct nl_sock *sock,
         i = seq - base_seq;
         txn = transactions[i];
 
+        const char *err_msg = NULL;
         /* Fill in the results for 'txn'. */
-        if (nl_msg_nlmsgerr(buf_txn->reply, &txn->error)) {
+        if (nl_msg_nlmsgerr(buf_txn->reply, &txn->error, &err_msg)) {
+            if (txn->error) {
+                VLOG_DBG_RL(&rl, "received NAK error=%d - %s",
+                            txn->error,
+                            err_msg ? err_msg : ovs_strerror(txn->error));
+            }
             if (txn->reply) {
                 ofpbuf_clear(txn->reply);
             }
-            if (txn->error) {
-                VLOG_DBG_RL(&rl, "received NAK error=%d (%s)",
-                            txn->error, ovs_strerror(txn->error));
-            }
         } else {
             txn->error = 0;
             if (txn->reply && txn != buf_txn) {
@@ -999,7 +1007,7 @@ nl_sock_transact_multiple__(struct nl_sock *sock,
             /* Handle errors embedded within the netlink message. */
             ofpbuf_use_stub(&tmp_reply, reply_buf, sizeof reply_buf);
             tmp_reply.size = sizeof reply_buf;
-            if (nl_msg_nlmsgerr(&tmp_reply, &txn->error)) {
+            if (nl_msg_nlmsgerr(&tmp_reply, &txn->error, NULL)) {
                 if (txn->reply) {
                     ofpbuf_clear(txn->reply);
                 }
@@ -1191,7 +1199,7 @@ nl_dump_refill(struct nl_dump *dump, struct ofpbuf *buffer)
         }
     }
 
-    if (nl_msg_nlmsgerr(buffer, &error) && error) {
+    if (nl_msg_nlmsgerr(buffer, &error, NULL) && error) {
         VLOG_INFO_RL(&rl, "netlink dump request error (%s)",
                      ovs_strerror(error));
         ofpbuf_clear(buffer);
@@ -1566,6 +1574,11 @@ do_lookup_genl_family(const char *name, struct nlattr **attrs,
         family_name = OVS_FLOW_FAMILY;
         family_version = OVS_FLOW_VERSION;
         family_attrmax = OVS_FLOW_ATTR_MAX;
+    } else if (!strcmp(name, OVS_METER_FAMILY)) {
+        family_id =  OVS_WIN_NL_METER_FAMILY_ID;
+        family_name = OVS_METER_FAMILY;
+        family_version = OVS_METER_VERSION;
+        family_attrmax = __OVS_METER_ATTR_MAX;
     } else if (!strcmp(name, OVS_WIN_NETDEV_FAMILY)) {
         family_id = OVS_WIN_NL_NETDEV_FAMILY_ID;
         family_name = OVS_WIN_NETDEV_FAMILY;
