@@ -55,15 +55,37 @@ nl_msg_genlmsghdr(const struct ofpbuf *msg)
     return ofpbuf_at(msg, NLMSG_HDRLEN, GENL_HDRLEN);
 }
 
+/* Parses the ext ack netlink attributes and, if successful, a pointer
+ * to the error message, if included, is stored in '*errmsg'. */
+static void
+nl_parse_ext_ack(const struct ofpbuf *msg, size_t offset, const char **errmsg)
+{
+    static const struct nl_policy policy[] = {
+        [NLMSGERR_ATTR_MSG]  = { .type = NL_A_STRING, .optional = true },
+    };
+    struct nlattr *attrs[ARRAY_SIZE(policy)];
+
+    if (!nl_policy_parse(msg, offset, policy, attrs, ARRAY_SIZE(policy))) {
+        VLOG_ERR_RL(&rl, "Failed to parse extended ack data");
+        return;
+    }
+
+    if (attrs[NLMSGERR_ATTR_MSG]) {
+        *errmsg = nl_attr_get_string(attrs[NLMSGERR_ATTR_MSG]);
+    }
+}
+
 /* If 'buffer' is a NLMSG_ERROR message, stores 0 in '*errorp' if it is an ACK
  * message, otherwise a positive errno value, and returns true.  If 'buffer' is
  * not an NLMSG_ERROR message, returns false.
  *
  * 'msg' must be at least as large as a nlmsghdr. */
 bool
-nl_msg_nlmsgerr(const struct ofpbuf *msg, int *errorp)
+nl_msg_nlmsgerr(const struct ofpbuf *msg, int *errorp, const char **attr_msg)
 {
-    if (nl_msg_nlmsghdr(msg)->nlmsg_type == NLMSG_ERROR) {
+    struct nlmsghdr *nlh = nl_msg_nlmsghdr(msg);
+
+    if (nlh->nlmsg_type == NLMSG_ERROR) {
         struct nlmsgerr *err = ofpbuf_at(msg, NLMSG_HDRLEN, sizeof *err);
         int code = EPROTO;
         if (!err) {
@@ -71,6 +93,15 @@ nl_msg_nlmsgerr(const struct ofpbuf *msg, int *errorp)
                         msg->size, NLMSG_HDRLEN + sizeof *err);
         } else if (err->error <= 0 && err->error > INT_MIN) {
             code = -err->error;
+            if (attr_msg && err->error != 0 &&
+                (nlh->nlmsg_flags & NLM_F_ACK_TLVS)) {
+                size_t offt =  NLMSG_HDRLEN + sizeof *err;
+
+                if (!(nlh->nlmsg_flags & NLM_F_CAPPED)) {
+                    offt += (err->msg.nlmsg_len - NLMSG_HDRLEN);
+                }
+                nl_parse_ext_ack(msg, offt, attr_msg);
+            }
         }
         if (errorp) {
             *errorp = code;
@@ -539,6 +570,13 @@ nl_msg_put_nested(struct ofpbuf *msg,
     nl_msg_end_nested(msg, offset);
 }
 
+/* Reset message size to offset. */
+void
+nl_msg_reset_size(struct ofpbuf *msg, size_t offset)
+{
+    msg->size = offset;
+}
+
 /* If 'buffer' begins with a valid "struct nlmsghdr", pulls the header and its
  * payload off 'buffer', stores header and payload in 'msg->data' and
  * 'msg->size', and returns a pointer to the header.
@@ -741,6 +779,20 @@ nl_attr_get_nested(const struct nlattr *nla, struct ofpbuf *nested)
     ofpbuf_use_const(nested, nl_attr_get(nla), nl_attr_get_size(nla));
 }
 
+/* Returns the Ethernet Address value in 'nla''s payload. */
+struct eth_addr
+nl_attr_get_eth_addr(const struct nlattr *nla)
+{
+    return NL_ATTR_GET_AS(nla, struct eth_addr);
+}
+
+/* Returns the Infiniband LL Address value in 'nla''s payload. */
+struct ib_addr
+nl_attr_get_ib_addr(const struct nlattr *nla)
+{
+    return NL_ATTR_GET_AS(nla, struct ib_addr);
+}
+
 /* Default minimum payload size for each type of attribute. */
 static size_t
 min_attr_len(enum nl_attr_type type)
@@ -757,6 +809,7 @@ min_attr_len(enum nl_attr_type type)
     case NL_A_FLAG: return 0;
     case NL_A_IPV6: return 16;
     case NL_A_NESTED: return 0;
+    case NL_A_LL_ADDR: return 6; /* ETH_ALEN */
     case N_NL_ATTR_TYPES: default: OVS_NOT_REACHED();
     }
 }
@@ -777,6 +830,7 @@ max_attr_len(enum nl_attr_type type)
     case NL_A_FLAG: return SIZE_MAX;
     case NL_A_IPV6: return 16;
     case NL_A_NESTED: return SIZE_MAX;
+    case NL_A_LL_ADDR: return 20; /* INFINIBAND_ALEN */
     case N_NL_ATTR_TYPES: default: OVS_NOT_REACHED();
     }
 }
