@@ -400,6 +400,7 @@ parse_body(struct ovsdb_log *file, off_t offset, unsigned long int length,
 
         chunk = MIN(length, sizeof input);
         if (fread(input, 1, chunk, file->stream) != chunk) {
+            sha1_final(&ctx, sha1);
             json_parser_abort(parser);
             return ovsdb_io_error(ferror(file->stream) ? errno : EOF,
                                   "%s: error reading %lu bytes "
@@ -658,7 +659,16 @@ ovsdb_log_write_and_free(struct ovsdb_log *log, struct json *json)
 struct ovsdb_error *
 ovsdb_log_commit_block(struct ovsdb_log *file)
 {
+#if (_POSIX_C_SOURCE >= 199309L || _XOPEN_SOURCE >= 500)
+    /* we do not check metadata - mtime, atime, anywhere, so we
+     * do not need to update it every time we sync the log.
+     * if the system supports it, the log update should be
+     * data only
+     */
+    if (file->stream && fdatasync(fileno(file->stream))) {
+#else
     if (file->stream && fsync(fileno(file->stream))) {
+#endif
         return ovsdb_io_error(errno, "%s: fsync failed", file->display_name);
     }
     return NULL;
