@@ -268,9 +268,7 @@ ovsdb_storage_read(struct ovsdb_storage *storage,
     struct json *schema_json = NULL;
     struct json *txn_json = NULL;
     if (storage->raft) {
-        bool is_snapshot;
-        json = json_nullable_clone(
-            raft_next_entry(storage->raft, txnid, &is_snapshot));
+        json = raft_next_entry(storage->raft, txnid);
         if (!json) {
             return NULL;
         } else if (json->type != JSON_ARRAY || json->array.n != 2) {
@@ -509,7 +507,11 @@ schedule_next_snapshot(struct ovsdb_storage *storage, bool quick)
 
         long long int now = time_msec();
         storage->next_snapshot_min = now + base + random_range(range);
-        storage->next_snapshot_max = now + 60LL * 60 * 24 * 1000; /* 1 day */
+        if (!quick) {
+            long long int one_day = 60LL * 60 * 24 * 1000;
+
+            storage->next_snapshot_max = now + one_day;
+        }
     } else {
         storage->next_snapshot_min = LLONG_MAX;
         storage->next_snapshot_max = LLONG_MAX;
@@ -517,7 +519,7 @@ schedule_next_snapshot(struct ovsdb_storage *storage, bool quick)
 }
 
 bool
-ovsdb_storage_should_snapshot(const struct ovsdb_storage *storage)
+ovsdb_storage_should_snapshot(struct ovsdb_storage *storage)
 {
     if (storage->raft || storage->log) {
         /* If we haven't reached the minimum snapshot time, don't snapshot. */
@@ -546,6 +548,15 @@ ovsdb_storage_should_snapshot(const struct ovsdb_storage *storage)
         }
 
         if (!snapshot_recommended) {
+            if (storage->raft) {
+                /* Re-scheduling with a quick retry in order to avoid condition
+                 * where all the raft servers passed the minimal time already,
+                 * but the log didn't grow a lot, so they are all checking on
+                 * every iteration.  This will randomize the time of the next
+                 * attempt, so all the servers will not start snapshotting at
+                 * the same time when the log reaches a critical size. */
+                schedule_next_snapshot(storage, true);
+            }
             return false;
         }
 
@@ -565,7 +576,7 @@ ovsdb_storage_should_snapshot(const struct ovsdb_storage *storage)
 static struct ovsdb_error * OVS_WARN_UNUSED_RESULT
 ovsdb_storage_store_snapshot__(struct ovsdb_storage *storage,
                                const struct json *schema,
-                               const struct json *data)
+                               const struct json *data, uint64_t index)
 {
     if (storage->raft) {
         struct json *entries = json_array_create_empty();
@@ -576,7 +587,7 @@ ovsdb_storage_store_snapshot__(struct ovsdb_storage *storage,
             json_array_add(entries, json_clone(data));
         }
         struct ovsdb_error *error = raft_store_snapshot(storage->raft,
-                                                        entries);
+                                                        entries, index);
         json_destroy(entries);
         return error;
     } else if (storage->log) {
@@ -600,10 +611,11 @@ ovsdb_storage_store_snapshot__(struct ovsdb_storage *storage,
 struct ovsdb_error * OVS_WARN_UNUSED_RESULT
 ovsdb_storage_store_snapshot(struct ovsdb_storage *storage,
                              const struct json *schema,
-                             const struct json *data)
+                             const struct json *data, uint64_t index)
 {
     struct ovsdb_error *error = ovsdb_storage_store_snapshot__(storage,
-                                                               schema, data);
+                                                               schema, data,
+                                                               index);
     bool retry_quickly = error != NULL;
     schedule_next_snapshot(storage, retry_quickly);
     return error;
@@ -627,7 +639,7 @@ ovsdb_storage_write_schema_change(struct ovsdb_storage *storage,
                                           prereq, &result);
         json_destroy(txn_json);
     } else if (storage->log) {
-        w->error = ovsdb_storage_store_snapshot__(storage, schema, data);
+        w->error = ovsdb_storage_store_snapshot__(storage, schema, data, 0);
     } else {
         /* When 'error' and 'command' are both null, it indicates that the
          * command is complete.  This is fine since this unbacked storage drops
